 
#include "APP001V3_LCD.H"
#include <xc.h>
// #include <delay.h>
//
// Defines for I/O ports that provide LCD data & control
// PORTD[0:3]-->DB[4:7]: Higher order 4 lines data bus with bidirectional
//					  : DB7 can be used as a BUSY flag
// PORTA,1 --> [E] : LCD operation start signal control 
// PORTA,2 --> [RW]: LCD Read/Write control
// PORTA,3 --> [RS]: LCD Register Select control
//		      	   : "0" for Instrunction register (Write), Busy Flag (Read)
//				   : "1" for data register (Read/Write)
//
#define CPU_FREQ		64				// CPU speed is 16 Mhz !!
#define LCD_RS				LATDbits.LATD4		// The definition of control pins
#define LCD_RW			LATDbits.LATD5
#define LCD_E				LATAbits.LATA2
#define LCD_DATA			LATD			// PORTD[0:3] as LCD DB[4:7]
#define DIR_LCD_DATA	TRISD	


//  LCD Module commands
#define DISP_2Line_8Bit		0b00111000
#define DISP_2Line_4Bit		0b00101000
#define DISP_ON			0x00C		// Display on
#define DISP_ON_C			0x00E		// Display on, Cursor on
#define DISP_ON_B			0x00F		// Display on, Cursor on, Blink cursor
#define DISP_OFF			0x008		// Display off
#define CLR_DISP			0x001		// Clear the Display
#define ENTRY_INC			0x006		//
#define ENTRY_INC_S		0x007		//
#define ENTRY_DEC		0x004		//
#define ENTRY_DEC_S		0x005		//
#define DD_RAM_ADDR	0x080		// Least Significant 7-bit are for address
#define DD_RAM_UL		0x080		// Upper Left coner of the Display	
		
unsigned char 	Temp_CMD ;
unsigned char 	Str_Temp ;
unsigned char	Out_Mask ;	
unsigned char	Temp_LCD_DATA ;	
static unsigned char Disable_Zero = 1;

void openLCD(void)

{
	ADCON1=(ADCON1 & 0xF0) | 0b00001110;	// Set AN0 for analog input
 	LCD_E=0;									
	Temp_LCD_DATA = LCD_DATA ;	
	
	LCD_DATA = 0x00;							// LCD DB[4:7] & RS & R/W --> Low

	DIR_LCD_DATA = 0x00;					// LCD DB[4:7} & RS & R/W are output function
	TRISAbits.TRISA2=0;						// Set E pin as output

	LCD_DATA = 0b00000011 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	LCD_DATA = 0b00000011 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	LCD_DATA = 0b00000011 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	LCD_DATA = 0b00000010 ;
	LCD_CMD_W_Timing() ;
	LCD_L_Delay() ;

	WriteCmdLCD(DISP_2Line_4Bit) ;
	LCD_S_Delay() ;

	WriteCmdLCD(DISP_ON) ;
	LCD_S_Delay() ;

	WriteCmdLCD(ENTRY_INC) ;
	LCD_S_Delay() ;

	WriteCmdLCD(CLR_DISP) ;
	LCD_L_Delay() ;

	LCD_DATA = Temp_LCD_DATA ;			
}

//*********************************************
//     _    ______________________________
// RS  _>--<______________________________
//     _____
// RW       \_____________________________
//                  __________________
// E   ____________/                  \___
//     _____________                ______
// DB  _____________>--------------<______
//***********************************************
// Write Command to LCD module
//
void WriteCmdLCD( unsigned char LCD_CMD) 
{

	Temp_LCD_DATA = LCD_DATA ;	

	Temp_CMD = (LCD_CMD & 0xF0)>>4 ;			// Send high nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_CMD_W_Timing () ;

	Temp_CMD = LCD_CMD & 0x0F ;				// Send low nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_CMD_W_Timing () ;
	LCD_DATA = Temp_LCD_DATA ;
	LCD_S_Delay() ;							// Delay 100uS for execution

}

//***********************************************
// Write Data to LCD module
//
void WriteDataLCD( unsigned char LCD_CMD) 
{
	
	Temp_LCD_DATA = LCD_DATA ;

	Temp_CMD = (LCD_CMD & 0xF0)>>4 ;			// Send high nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_DAT_W_Timing () ;

	Temp_CMD = LCD_CMD & 0x0F ;				// Send low nibble to LCD bus
	LCD_DATA= (LCD_DATA & 0xF0)|Temp_CMD ;
	LCD_DAT_W_Timing () ;
	LCD_DATA = Temp_LCD_DATA ;
	LCD_S_Delay() ;							// Delay 100uS for execution


}

void putcLCD(unsigned char LCD_Char)
{
	WriteDataLCD(LCD_Char) ;

}
void LCD_CMD_W_Timing( void )
{
	LCD_RS = 0 ;	// Set for Command Input
	Nop();
	LCD_RW = 0 ;
	Nop();
	LCD_E = 1 ;
	Nop();
	Nop();
	LCD_E = 0 ;
}

void LCD_DAT_W_Timing( void )
{
	LCD_RS = 1 ;	// Set for Data Input
	Nop();
	LCD_RW = 0 ;
	Nop();
	LCD_E = 1 ;
	Nop();
	Nop();
	LCD_E = 0 ;
}

//***********************************************
//     Set Cursor position on LCD module
//			CurY = Line (0 or 1)
//      	CurX = Position ( 0 to 15)
//
void setCursorLCD(unsigned char CurX, unsigned char CurY)
{
	WriteCmdLCD( 0x80 + CurY * 0x40 + CurX) ;
	LCD_S_Delay() ;
}

//***********************************************
//    Put a ROM string to LCD Module
//
void putrsLCD( const char *Str )
{
   while (1)
   {
	Str_Temp = *Str ;

		if (Str_Temp != 0x00 )
		   {
			WriteDataLCD(Str_Temp) ;
			Str ++ ;
		   }
		else
			return ;
   }
}

//***********************************************
//    Put a RAM string to LCD Module
//
void putsLCD( char *Str)
{
   while (1)
   {
	Str_Temp = *Str ;

		if (Str_Temp != 0x00 )
		   {
			WriteDataLCD(Str_Temp) ;
			Str ++ ;
		   }
		else
			return ;
   }
}


void putHexCharLCD(unsigned char HEX_Val)
{
	unsigned char Temp_HEX ;

	Temp_HEX = (HEX_Val >> 4) & 0x0f ;

	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;

	Temp_HEX = HEX_Val  & 0x0f ;
	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;
}

void putHexIntLCD(unsigned int HEX_Val)
{
	unsigned char Temp_HEX ;

 	Temp_HEX = (HEX_Val >> 12) & 0x000f ;

	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;   
    
	Temp_HEX = (HEX_Val >> 8) & 0x000f ;

	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;    
    
	Temp_HEX = (HEX_Val >> 4) & 0x000f ;

	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;

	Temp_HEX = HEX_Val  & 0x000f ;
	if ( Temp_HEX > 9 )Temp_HEX += 0x37 ;
    else Temp_HEX += 0x30 ;

	WriteDataLCD(Temp_HEX) ;
}

unsigned char LCD_BCD_Regulate( unsigned char BCD )
{
		if ( BCD == 0 ) 
		{
			if ( Disable_Zero == 1 ) 
				return ' ';
			else
				return '0';
		}

		else 
		{
			Disable_Zero = 0;
			return ( BCD += '0' );
		}		
}

void putNumberLCD( unsigned int Number , unsigned char Digit )
{
	unsigned char Temporary;
	Disable_Zero = 1;

	switch ( Digit )
	{
		default:
		case 4:
			Temporary = LCD_BCD_Regulate( ( Number % 10000 ) / 1000 );
			WriteDataLCD( Temporary );

		case 3:
			Temporary = LCD_BCD_Regulate( ( Number % 1000 ) / 100 );
			WriteDataLCD( Temporary );

		case 2:
			Temporary = LCD_BCD_Regulate( ( Number % 100 ) / 10 );
			WriteDataLCD( Temporary );

		case 1:
			Disable_Zero = 0;
			Temporary = LCD_BCD_Regulate( Number % 10 );
			WriteDataLCD( Temporary );
	}
}

// *********************************************************************************
// Delay for atleast 10 ms 
// *********************************************************************************
void LCD_L_Delay(void)
{
	_delay(CPU_FREQ * 100) ;		
    _delay(CPU_FREQ * 100) ;
	_delay(CPU_FREQ * 100) ;		
    _delay(CPU_FREQ * 100) ;    
    _delay(CPU_FREQ * 100) ;      
}

// *********************************************************************************
// Delay for 100 us
// *********************************************************************************
void LCD_S_Delay(void)
{
	_delay(CPU_FREQ * 10) ;	
}
